;;; personal.el --- Setup modes, hooks and other personal stuff
;; Copyright (C) 2000-2004 Stefan Kamphausen

;; Author: Stefan Kamphausen <http://www.skamphausen.de>
;; Time-stamp: <22-May-2006 16:59:55 ska>

;; Keywords: 
;; This file is not part of XEmacs.

;; This code is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This code is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;; General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with this code; see the file COPYING.  If not, write to the Free
;; Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
;; 02111-1307, USA.

;;; not Synched up with: (X)Emacs

;;; Commentary:
;; This is The Next Generation of my XEmacs customization.
;; What formerly had been ~/.xemacs-personal now should be in this
;; file. It sets up all the major modes and their hooks, loads several
;; files (keys and utils and other non-standard stuff), sets the paths
;; and more. Just have a look.
;; Please note that this is a folded file, thus it hides almost
;; everything when you first open it. Simply right-click on the points
;; where you see three braces ('{') and that tree should appear. Or
;; use M-x fold-open-buffer (C-c @ C-o) to show everything.


;; To use:
;; put in your .emacs (or ~/.xemacs/init.el for recent XEmacsen):
;;
;;(defvar my-xemacs-dir
;;  (expand-file-name "~/.xemacs/my")
;;"The directory where all the XEmacs configuration (and more) goes.
;;This contains all my files, not those of others.")
;;
;;(cond
;; ((string-match "XEmacs" emacs-version)
;;  (load (concat my-xemacs-dir "/config/personal.el") nil nil t)))
;;
;; Actually this file is part of a large-scale setup I wrote and I
;; tried to create it as generically as possible so that you can use
;; it yourself with only a few changes (which probably will be in
;; user.el).

;;; Code:


;;============================================================================
;;{{{ PATHs, Filenames and Modules
;;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

;;{{{ Define some directories containing our own stuff
(defvar my-lisp-dir
  (concat my-xemacs-dir "/lisp")
  "The personal lisp directory. All self-written files and files from
the web goe here. Larger packages like e.g. VM or PSGML are installed
in subdirs."
  )

(defvar my-config-dir
  (concat my-xemacs-dir "/config")
  "This directory contains the configuration files. Keybindings as well
as local/personal settings plus init files of other packages
e.g. vm-init-file"
  )

(defvar my-templates-dir
  (concat my-xemacs-dir "/templates")
  "This directory contains all templates, inserting features and
skeletons I use together with XEmacs."
  )
(defvar my-data-dir
  (concat my-xemacs-dir "/data")
  "This directory contains all data XEmacs likes to store."
  )
;;}}}
;;{{{ Setting paths and list
(setq data-directory-list
      (append
       (list
        (concat my-xemacs-dir "/etc")
        )
       data-directory-list
       ))

(setq load-path
      (append
       (list
        my-config-dir
        my-templates-dir
        ;;(expand-file-name "~/src/elisp/mtorus/")
        my-lisp-dir)
       load-path
       ))
;;}}}
;;{{{ Loading the site files
;; (hopefully) the only file with user specific informations
(load "user" nil nil)
;(load "redhat90" nil nil) ;; GRRRR!!!
(setq my-host-file
      (concat my-config-dir "/"
              (replace-in-string (exec-to-string "hostname")
                                 "\n" "")
              ".el"))
(if (file-readable-p my-host-file)
    (load my-host-file nil nil))
(setq my-domain-file
      (concat my-config-dir "/"
              (replace-in-string (exec-to-string "domainname")
                                 "\n" "")
              ".el"))
(if (file-readable-p my-domain-file)
    (load my-domain-file nil nil))
;;}}}
;;{{{ The settings of the new filenames for some packages
(setq vm-init-file            (concat my-config-dir "/vm-init.el"))
(setq gnus-init-file          (concat my-config-dir "/gnus-init.el"))
(setq gnus-directory          (expand-file-name     "~/contact/gnus/"))
(setq message-directory       (expand-file-name     "~/contact/gnus/"))
(setq save-place-file         (concat my-data-dir   "/places.el"))
(setq recent-files-save-file  (concat my-data-dir   "/recent-files.el"))
(setq picklist-save-file      (concat my-data-dir   "/picklist.el"))
(setq bbdb-file               (concat my-data-dir   "/bbdb"))
(setq w3-hotlist-file         (concat my-data-dir   "/w3-hotlist"))
(setq w3-configuration-directory (concat my-data-dir "/w3"))
(setq eshell-directory-name   (concat my-data-dir   "/eshell/"))
(setq shadow-info-file        (concat my-data-dir   "/shadows"))
(setq shadow-todo-file        (concat my-data-dir   "/shadow_todo"))
(setq ctypes-file-name        (concat my-data-dir   "/ctypes.el"))
(setq bookmark-default-file   (concat my-data-dir   "/bookmarks.el"))
(setq eudc-options-file       (concat my-data-dir   "/eudc-options.el"))
(setq auto-save-directory     (concat my-data-dir   "/autosave/"))
(setq auto-save-list-file-prefix (concat my-data-dir   "/autosave/saves-"))
;;}}}
;;{{{ Our own save require defun
;; inspired by Init-safe-require from the new sample.init.el file
(defun ska-safe-require (feat)
"Try to REQUIRE the specified feature.  Errors occurring are silenced.
\(Perhaps in the future there will be a way to get at the error.)
Returns t if the feature was successfully required."
  (condition-case nil
      (require feat)
    (error (display-warning 'personal-load (format "Unable to load %s " feat)))))
;;}}}
;;{{{ Now load the keybindings (they require the utils then)
(ska-safe-require 'ska-global-keys)
(ska-safe-require 'ska-local-keys)
;;}}}

;;}}}
;;============================================================================

;;============================================================================
;;{{{ MAJOR MODES
;;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;;{{{ Early Stuff
;; This section contains things that I want to enable globally or can't find a
;; way to start later and work correctly (like setnu).
(load "setnu")
(turn-on-font-lock)
(pending-delete-on)
;;}}}
;;============================================================================
;;{{{ Text Mode
(ska-safe-require 'filladapt)
(setq auto-mode-alist
      (append '(("README" . text-mode)) auto-mode-alist))

(add-hook 'text-mode-hook
          '(lambda () 
             (auto-fill-mode 1)
             (turn-on-filladapt-mode)
             ;;(abbrev-mode 1)
             ;;(if (file-readable-p "~/.abbrev_defs")
             ;;  (read-abbrev-file "~/.abbrev_defs"))
             ))
;;}}}
;;----------------------------------------------------------------------------
;;{{{ Perl Mode: cperl
(setq cperl-mode-hook 
      '(lambda ()
         ;; my keybindings
         (ska-coding-keys cperl-mode-map)
         (ska-cperl-mode-keys)
         (auto-fill-mode 1)
         (turn-on-setnu-mode)
         (turn-on-font-lock)
         ;; if you don't want tabs
         (setq indent-tabs-mode nil)
         ;; full featured mode:
         ;;(setq cperl-hairy t)
         ;; alternatively:
         ;;(setq cperl-auto-newline-after-colon t)
         ;;(setq cperl-electric-parens "({[")
         (setq cperl-auto-newline nil)
         (setq cperl-electric-linefeed nil)
         (setq cperl-electric-parens nil)
         (setq cperl-electric-lbrace-space t)
         (setq cperl-electric-keywords nil)
         (setq cperl-mode-abbrev-table nil)
         (setq cperl-lazy-help-time 1)
         (setq cperl-help t)
         (setq cperl-pod-here-fontify nil)
         (setq cperl-highlight-variables-indiscriminately t)
         (setq Manual-buffer-view-mode 1) ; for a separate buffer
         (make-local-hook 'write-contents-hooks)
         (add-hook 'write-contents-hooks #'ska-untabify
                   nil t)
         ))

;; from newsgroups (with title "trailing spaces in Cperl Mode")
;; Here's my solution:
;;
;; M-X customize RET cperl-faces RET
;; Find "Invalid Face".  Set its value to: '(quote default)
;; Set, Save, Done
;;
;; I don't know if this is the "correct" way to accomplish this, or even
;; the best way, but it seems to work.  It seems odd there's not a
;; easier way to accomplish this, since everything else about cperl-mode
;; seems customizable. 
(defun strip-end-of-line-whitespace ()
  (interactive)
  
  (let ((count 0))
    (save-excursion
      (goto-char (point-min))
      (let ((case-replace nil)
            (case-fold-search nil))
        (while (re-search-forward "[ \t]+$" nil t)
          (replace-match "" t t)
          (incf count))))
    (when (interactive-p)
      (message "Stripped %d line%s." 
               count 
               (if (= count 1) "" "s")))))


;;}}}
;;----------------------------------------------------------------------------
;;{{{ Ruby
;; this code is from inf-ruby.el
(autoload 'ruby-mode "ruby-mode"
  "Mode for editing ruby source files")
(setq auto-mode-alist
      (append '(("\\.rb$" . ruby-mode)) auto-mode-alist))
(setq interpreter-mode-alist (append '(("ruby" . ruby-mode))
                                     interpreter-mode-alist))
(autoload 'run-ruby "inf-ruby"
  "Run an inferior Ruby process")
(autoload 'inf-ruby-keys "inf-ruby"
  "Set local key defs for inf-ruby in ruby-mode")
(autoload 'rubydb "rubydb3x"
  "Run ruby debugger")
(add-hook 'ruby-mode-hook
          '(lambda ()
             (ska-coding-keys ruby-mode-map)
             (ska-ruby-mode-keys)
             (auto-fill-mode 1)
             (turn-on-filladapt-mode)
             (turn-on-setnu-mode)
             (turn-on-font-lock)
             (inf-ruby-keys)
             ))
;;}}}
;;----------------------------------------------------------------------------
;;{{{ HTML PSGML Mode
;; not working in hook?
(setq html-auto-sgml-entity-conversion t)
;;(setq sgml-data-directory (concat my-data-dir "/psgml"))
(add-hook 'html-mode-hook
          '(lambda ()
             (auto-fill-mode 1)
             (setq sgml-insert-missing-element-comment nil)
             (setq sgml-set-face nil)
             (setq sgml-markup-faces nil)
             (setq fill-column 100)
             (setq html-helper-use-expert-menu t)
             (setq html-helper-do-write-file-hooks t)
             (add-to-list 'sgml-catalog-files
                          (concat sgml-data-directory "/CATALOG"))
             ))
;;}}}
;;----------------------------------------------------------------------------
;;{{{ CSS - Cascading Style Sheets
;; needs css-mode.el from
;; http://www.garshol.priv.no/download/software/css-mode/
(autoload 'css-mode "css-mode")
(setq auto-mode-alist       
      (cons '("\\.css\\'" . css-mode) auto-mode-alist))
;;}}}
;;----------------------------------------------------------------------------
;;{{{ SGML, XML and XSL Mode
(autoload 'sgml-mode "psgml" "Major mode to edit SGML files." t)
(autoload 'xml-mode "psgml" "Major mode to edit XML files." t)
(setq auto-mode-alist
      (append
       '(("\\.xml$" .xml-mode))
       auto-mode-alist))

;; docbook still isn't working??
(add-hook 'sgml-mode-hook
          '(lambda ()
;            ;;(setq sgml-catalog-files '("/usr/share/xemacs/21.1.8/etc/psgml/CATALOG"))
             ;; correct display of ISO8859-1 chars (umlauts etc)
             (load "iso-sgml")
             (setq isosgml-modes-list
                   (append
                    '(xml-mode)
                    isosgml-modes-list))
             
             (ska-sgml-keys)
             (setq sgml-auto-insert-required-elements t)
             (setq sgml-insert-end-tag-on-new-line t)
             (setq sgml-insert-missing-element-comment nil)
             ;; (setq sgml-custom-markup '(("menustring" "insert\rstring")))
             (setq sgml-indent-data t)
             ;;display element name in modeline.
             ;; this makes problems when the DTD isn't parsed yet:
             ;(setq sgml-live-element-indicator t)
                
             ;;;;; Faces.
             ;; uncommenting the following because fontlock now works
;;             (setq sgml-set-face t)   ;; whether PSGML colorizes buffer 
;;             (setq sgml-auto-activate-dtd t) ;; autom parse DTD
;;             (setq sgml-use-text-properties t) 
;;             (setq sgml-markup-faces
;;                   '((start-tag     . font-lock-keyword-face)
;;                     (end-tag   . font-lock-keyword-face)
;;                     (comment   . font-lock-comment-face)
;;                     (pi        . font-lock-string-face)
;;                     (sgml      . font-lock-reference-face)
;;                     (doctype   . font-lock-variable-name-face)
;;                     (entity    . font-lock-function-name-face)
;;                     (shortref  . font-lock-type-face)))
             ;; if you don't like the default font-lock faces make
             ;; your own:
;;             (make-face 'sgml-comment-face)
;;             (make-face 'sgml-doctype-face)
;;             (make-face 'sgml-end-tag-face)
;;             (make-face 'sgml-entity-face)
;;             (make-face 'sgml-ignored-face)
;;             (make-face 'sgml-ms-end-face)
;;             (make-face 'sgml-ms-start-face)
;;             (make-face 'sgml-pi-face)
;;             (make-face 'sgml-sgml-face)
;;             (make-face 'sgml-short-ref-face)
;;             (make-face 'sgml-start-tag-face)
         
;;             (set-face-foreground 'sgml-comment-face "dimgrey")
;;             (set-face-foreground 'sgml-doctype-face "maroon")
;;             (set-face-foreground 'sgml-end-tag-face "blue2")
;;             (set-face-foreground 'sgml-entity-face "seagreen")
;;             (set-face-foreground 'sgml-ignored-face "maroon")
;;             (set-face-background 'sgml-ignored-face "gray90")
;;             (set-face-foreground 'sgml-ms-end-face "maroon")
;;             (set-face-foreground 'sgml-ms-start-face "maroon")
;;             (set-face-foreground 'sgml-pi-face "maroon")
;;             (set-face-foreground 'sgml-sgml-face "maroon")
;;             (set-face-foreground 'sgml-short-ref-face "goldenrod")
;;             (set-face-foreground 'sgml-start-tag-face "blue2")
         
;;           (setq-default sgml-markup-faces
;;                         '((comment . sgml-comment-face)
;;                           (doctype . sgml-doctype-face)
;;                           (end-tag . sgml-end-tag-face)
;;                           (entity . sgml-entity-face)
;;                           (ignored . sgml-ignored-face)
;;                           (ms-end . sgml-ms-end-face)
;;                           (ms-start . sgml-ms-start-face)
;;                           (pi . sgml-pi-face)
;;                           (sgml . sgml-sgml-face)
;;                           (short-ref . sgml-short-ref-face)
;;                           (start-tag . sgml-start-tag-face)))
             ))


;; DocBook IDE mode
;; from http://www.nwalsh.com/emacs/docbookide/
(autoload 'docbook-mode "docbookide" "Major mode for DocBook documents." t)
;(add-hook 'docbook-mode-hook
;          'turn-on-font-lock)
(setq auto-mode-alist
      (append
       (list
        '("\\.docbook" . docbook-mode))
       auto-mode-alist))


;; XSL mode
(autoload 'xsl-mode "xslide" "Major mode for XSL stylesheets." t)
(add-hook 'xsl-mode-hook
          '(lambda ()
             ;; xslide uses font-lock 
             (setq-default sgml-set-face nil)
             (setq sgml-markup-faces nil)
             (turn-on-font-lock)
             ))

(setq auto-mode-alist
      (append
       (list
        '("\\.fo$" . xsl-mode)
        '("\\.xsl$" . xsl-mode))
       auto-mode-alist))

;; sxml-mode -- better?
;(autoload 'sxml-mode "sxml-mode" "Major mode for editing XML documents." t)
;(setq auto-mode-alist
;      (append
;       '((".*\.xml$" . sxml-mode )
;        (".*\.xsl$" . sxml-mode))
;       auto-mode-alist))
;; and there is a package xxml.el which fontifies XML docs, but only
;; in gnu emacs?
;;}}}
;;----------------------------------------------------------------------------
;;{{{ DTD Mode
(autoload 'dtd-mode "tdtd" "Major mode for SGML and XML DTDs." t)
(autoload 'dtd-etags "tdtd"
  "Execute etags on FILESPEC and match on DTD-specific regular expressions."
  t)
(autoload 'dtd-grep "tdtd" "Grep for PATTERN in files matching FILESPEC." t)

;; Turn on font lock when in DTD mode
(add-hook 'dtd-mode-hooks
          'turn-on-font-lock)

(setq auto-mode-alist
      (append
       (list
        '("\\.dcl$" . dtd-mode)
        '("\\.dec$" . dtd-mode)
        '("\\.dtd$" . dtd-mode)
        '("\\.ele$" . dtd-mode)
        '("\\.ent$" . dtd-mode)
        '("\\.mod$" . dtd-mode))
       auto-mode-alist))
;;}}}
;;----------------------------------------------------------------------------
;;{{{ DQS Stuff
(setq auto-mode-alist
      (append
       '(("\\.dqs$" .ksh-mode))
       auto-mode-alist))
;;}}}
;;----------------------------------------------------------------------------
;;{{{ Postscript
;; must require it, when autoloading, my ps-postscript-command is overwritten
;;(autoload 'postscript-mode "postscript.el" "" t)
(ska-safe-require 'postscript)
(setq auto-mode-alist
      (cons '("\\.e?ps$".postscript-mode) auto-mode-alist))
(defconst ps-postscript-command '("gv" "-"))
;;}}}
;;----------------------------------------------------------------------------
;;{{{ BraveGNUWorldMode
(autoload 'bravegnuworld-mode "bravegnuworld")
(setq auto-mode-alist
      (append
       '(("BraveGNUWorld.*\.html" . bravegnuworld-mode))
       auto-mode-alist))
;;}}}
;;----------------------------------------------------------------------------
;;{{{ eLisp Mode

(add-hook 'emacs-lisp-mode-hook
          '(lambda ()
             ;; meine Keybindings
             (ska-coding-keys emacs-lisp-mode-map)
             (ska-elisp-mode-keys)
             (auto-fill-mode 1)
             (turn-on-font-lock)
             (turn-on-setnu-mode)
             (turn-on-filladapt-mode)
             (turn-on-eldoc-mode)
             (make-local-hook 'write-contents-hooks)
             (add-hook 'write-contents-hooks #'ska-untabify
                       nil t)
             ))

(add-hook 'lisp-interaction-mode-hook
          '(lambda ()
             (ska-coding-keys emacs-lisp-mode-map)
             (ska-elisp-mode-keys)
             (setq mode-name "LispI")
             (turn-on-eldoc-mode)
             (turn-on-font-lock)
             (turn-on-setnu-mode)
             (make-local-hook 'write-contents-hooks) 
             ;;(add-hook 'write-contents-hooks 'ska-untabify)
             ))
;;}}}
;;----------------------------------------------------------------------------
;;----------------------------------------------------------------------------
;;{{{ Common Lisp Mode
;; Taken from CHBs chb-modes.el (ilisp with cmucl)
;;(setq cmulisp-program "/tools/common/cmucl2/bin/lisp")
;(setq cmulisp-local-source-directory "/tools/common/cmucl/src")
;;(setq common-lisp-hyperspec-root "http://einstein.nsi-hsn.de/externalDocu/Common_Lisp_HyperSpec/")

;(autoload 'run-ilisp   "ilisp" "Select a new inferior Lisp." t)
;(autoload 'cmulisp     "ilisp" "Inferior CMU Common Lisp." t)
;(defun cmu ()
;  (interactive)
;  (cd command-line-default-directory)
;  (cmulisp))

;(add-hook 'lisp-mode-hook #'(lambda ()
;                              (require 'ilisp)
;                              (local-set-key '[(return)]
;                                             #'(lambda ()
;                                                 (interactive "*")
;                                                 (newline-and-indent-lisp)
;                                                 (move-to-column (current-indentation))))))

;(add-hook 'ilisp-load-hook
;          #'(lambda ()
;              (setq ilisp-*prefix* "\C-c")
;              (setq ilisp-*use-fsf-compliant-keybindings* t)
;              (setq ilisp-*enable-cl-easy-menu-p* t)
;              (setq ilisp-*enable-scheme-easy-menu-p* t)
;              (setq lisp-no-popper t)
;              (setq ilisp-motd nil)
;              (if (eq window-system 'x)
;                  (setq lisp-font-lock-keywords lisp-font-lock-keywords-2))
;              (defkey-ilisp '[(control c) (control h)] 'hyperspec-lookup)
;              (defkey-ilisp '[(control b) (control b)] #'comint-interrupt-subjob)
;              (add-hook 'ilisp-init-hook
;                        #'(lambda ()
;                            (default-directory-lisp ilisp-last-buffer)
;                            ;; this is sooo ugly, but
;                            ;; comint-output-filter-functions and
;                            ;; similar stuff just DID NOT WORK.
;                            (require 'advice)
;                            (require 'comint)
;                            (defadvice comint-insert (after ilisp-comint-insert activate)
;                              (save-selected-window
;                                (select-window
;                                 (get-buffer-window (ilisp-buffer) t)
;                                 t)
;                                (goto-char (point-max))
;                                (recenter -1)))
                            
;                            (when (file-readable-p "./.ilisp-inferior-lisp-startup.lisp")
;                              (ilisp-send "(load \"./.ilisp-inferior-lisp-startup.lisp\")")
;                              (message ".ilisp-inferior-lisp-startup.lisp loaded"))))
;              (add-hook 'ilisp-mode-hook
;                        #'(lambda ()
;                            (defkey-ilisp '[(control c) (control c)] 
;                              #'compile-defun-and-go-lisp)
;;                            (defkey-ilisp
;;                              '[(f11)]
;;                              #'(lambda ()
;;                                  (interactive)
;;                                  (with-current-buffer (ilisp-buffer)
;;                                    (comint-send-eof)
;;                                    (while (comint-check-proc (current-buffer))
;;                                      (sleep-for 0.1))
;;                                    (cmulisp))))
;                            (setq truncate-lines nil)))
;              ))

;(add-hook 'cmulisp-hook
;          #'(lambda ()
;              (setq ilisp-init-binary-extension "x86f")
;              (setq ilisp-init-binary-command "(progn \"x86f\")")
;              (setq ilisp-binary-extension "x86f")
;              (setq comint-prompt-regexp
;                    "^\\([0-9]+\\]+\\|\\*\\|[-a-zA-Z0-9]*\\[[0-9]+\\]:\\|[-A-Z0-9]*>\\) ")
;              ))

;; Now switch to SLIME:
(ska-safe-require 'slime)
(ska-safe-require 'clisp-indent)

;; my own suggestion is in official code now *happiness*
;;(ska-safe-require 'slime-hist)
;;(defadvice slime-quit-lisp (around slime-quit-lisp-with-history activate)
;;  (slime-repl-save-merged-history)
;;  ad-do-it)  

(setq common-lisp-hyperspec-root "/usr/share/doc/hyperspec-7.0/HyperSpec/")
;;(setq slime-complete-symbol-function 'slime-fuzzy-complete-symbol)
(setq slime-complete-symbol-function 'slime-complete-symbol*)
(setq slime-conservative-indentation nil)
(setq auto-mode-alist
      (append
       '(("\\.lsp$" . lisp-mode)
         ("\\.asd$" . lisp-mode)
         ("\\.cl$" . lisp-mode)
         ("\\.system$" . lisp-mode))
       auto-mode-alist))

(add-hook 'lisp-mode-hook
          (lambda ()
            (ska-coding-keys shared-lisp-mode-map)
            (ska-lisp-mode-keys)
            (slime-mode t)
            (auto-fill-mode 1)
            (turn-on-filladapt-mode)
            (setq fill-column 80)
            (turn-on-setnu-mode)
            (speedbar-add-supported-extension ".lisp")
            (make-local-hook 'write-contents-hooks)
            (add-hook 'write-contents-hooks #'ska-untabify
                      nil t)
            (setq indent-tabs-mode nil)
            (set (make-local-variable 'lisp-indent-function)
                 'common-lisp-indent-function)
            ))
(add-hook 'inferior-lisp-mode-hook
          (lambda ()
            (inferior-slime-mode t)))

(add-hook 'slime-repl-mode-hook
          (lambda ()
            (slime-repl-read-history)
            (ska-coding-keys shared-lisp-mode-map)
            (ska-lisp-mode-keys)))


;; Optionally, specify the lisp program you are using. Default is "lisp"
;;(setq inferior-lisp-program "/usr/bin/sbcl")
(setq inferior-lisp-program "/usr/bin/lisp")
(def-slime-start cmucl "/usr/bin/lisp")
(def-slime-start sbcl "/usr/bin/sbcl")

;;}}}
;;----------------------------------------------------------------------------
;;{{{ SQL Mode
(autoload 'sql-mode "sql" "SQL Editing Mode" t)
(setq auto-mode-alist
      (append
       '(("\\.sql$" . sql-mode))
       auto-mode-alist))
(add-hook 'sql-mode-hook
          '(lambda ()
             (ska-coding-keys sql-mode-map)
             (ska-sql-mode-keys)
             (sql-highlight-oracle-keywords)
             ))
;;}}}
;;----------------------------------------------------------------------------
;;{{{ C-MODE

(add-hook 'c++-mode-hook
          '(lambda ()
             ;; my keybindings
             (ska-coding-keys c++-mode-map)
             (ska-c-common-mode-keys c++-mode-map)
             (ska-c++-mode-keys)
             ))

(add-hook 'c-mode-hook
          '(lambda ()
             ;; my keybindings
             (ska-coding-keys c-mode-map)
             (ska-c-common-mode-keys c-mode-map)
             (ska-c-mode-keys)
             ))

(add-hook 'c-mode-common-hook
          '(lambda ()
             (auto-fill-mode 1)
             ;; with c-ignore-auto-fill we have indention only in comments :)
             (setq fill-column 70)
             ;; determines which context to show when matching paren off screen
             (setq paren-backwards-message t)
             ;; line numbers
             (turn-on-setnu-mode)
             ;; syntax highlighting
             (turn-on-font-lock)
             ;; mark the current line -- somehow it doesn't really
             ;; work together with setnu.el
             ;;(marky-mode) 
             ;; ctype dynamically scans files for new type definitions
             (ska-safe-require 'ctypes)
             ;; hereby we collect all we've seen
             (setq ctypes-write-types-at-exit t)
             ;; .. and read it
             (ctypes-read-file nil nil t t)
             ;; needed for ever collecting, too:
             (ctypes-auto-parse-mode 1)
             ;; testing the oo-browser from beopen.com
             ;;(load "br-start")
             (setq c-indent-comments-syntactically-p t)
             ;; testing the electric and hungry features...
             ;; (c-toggle-auto-hungry-state t)
             ;; ... no, I don't like em...
             (make-local-hook 'write-contents-hooks)
             (add-hook 'write-contents-hooks #'ska-untabify
                       nil t)
             (setq indent-tabs-mode nil)
             ))

;;}}}
;;---------------------------------------------------------------------------- 
;;{{{ JDE Java Development
;; If you want Emacs to defer loading the JDE until you open a 
;; Java file, edit the following line
(setq defer-loading-jde t)

(if defer-loading-jde
    (progn
      (autoload 'jde-mode "jde" "JDE mode." t)
      (setq auto-mode-alist
        (append
         '(("\\.java\\'" . jde-mode))
         auto-mode-alist)))
  (require 'jde))

;; Note that the customization for JDE is done via customize, because
;; JDE wants it that way
(defun my-jde-mode-hook ()
  (turn-on-setnu-mode)
  (turn-on-font-lock)
  (auto-fill-mode 1)
  (setq indent-tabs-mode nil)

  (setq c-basic-offset 2)

  (ska-coding-keys jde-mode-map)
  (ska-jde-mode-keys)
  (make-local-hook 'write-contents-hooks)
  (add-hook 'write-contents-hooks #'ska-untabify
            nil t))

(add-hook 'jde-mode-hook 'my-jde-mode-hook)


;;}}}
;;---------------------------------------------------------------------------- 
;;{{{ Gnuplot-Mode

;; gnuplot-mode.el is no standard
(autoload 'gnuplot-mode "gnuplot" "gnuplot major mode" t)
(autoload 'gnuplot-make-buffer "gnuplot" "open a buffer in gnuplot mode" t)
(setq auto-mode-alist
      (append
       '(("\\.gp$" . gnuplot-mode)
         ("\\.gnu$" . gnuplot-mode))
       auto-mode-alist))
(add-hook 'gnuplot-mode-hook
          '(lambda ()
             ;; meine Keybindings
             (ska-coding-keys gnuplot-mode-map)
             ))
;;}}}
;;----------------------------------------------------------------------------
;;{{{ LaTeX Handling
(ska-safe-require 'tex-site)

(setq TeX-parse-self t) ; Enable parse on load.
(setq TeX-auto-save t) ; Enable parse on save.
(setq-default TeX-master nil)

(add-hook 'LaTeX-mode-hook
          '(lambda ()
             ;; meine keybindings
             (ska-coding-keys TeX-mode-map)
             (ska-latex-mode-keys)
             (fume-add-menubar-entry)
             (setq fume-display-in-modeline-p nil)
             ;(setq ispell-parser 'tex)
             (LaTeX-math-mode)
             (turn-on-reftex)
;             (when (string-match "diary" buffer-file-name)
;               (turn-on-tex-diary))
             ))

(add-hook 'TeX-language-de-hook
          '(lambda ()
             (ispell-change-dictionary "deutsch")
             ))


;;(add-hook 'LaTeX-mode-hook
;;  '(lambda {}
;; set bibtex-file or environment variable described in
;; reftex-customize menu to make C-c [ (insert \cite) work
;;     ('turn-on-reftex)
;;     ))
;;}}}
;;---------------------------------------------------------------------------- 
;;{{{ BiTeX-Mode-Hook
 (add-hook 'BibTeX-mode-hook
          '(lambda ()
             (ska-bibtex-mode-keys)
             ))
;;}}}
;;---------------------------------------------------------------------------- 
;;{{{ terminal emulation
(defun bug-evil-term-process-coding-system ()
  "Fix a term bug; the `process-coding-system' should always be `binary'."
  (set-buffer-process-coding-system 'binary 'binary))

(add-hook 'term-exec-hook 'bug-evil-term-process-coding-system)
;;}}}
;;---------------------------------------------------------------------------- 
;;{{{ (Unix) Manual
(setq Manual-switches '("-a"))
;;}}}
;;---------------------------------------------------------------------------- 
;;{{{ Visiting files 

(add-hook 'find-file-hooks 
          '(lambda ()
             ;; this makes xemacs WAAY slow (at least in CC files)
             ;;(fume-add-menubar-entry)
             ;;(marky-mode) doesnt work correctly (yet)
             ;;(ska-fit-fill-column-to-frame)
             ;(ska-add-fixme-highlighting)
             (auto-insert)
             ))
;;}}}
;;----------------------------------------------------------------------------
;;{{{ Writing files
(add-hook 'write-file-hooks 
          '(lambda ()
             (time-stamp)
             ))
;;}}}
;;----------------------------------------------------------------------------
;;{{{ Info Mode
;; I'd like to read info in Helvetica, but I can't get it to work :(
;(add-hook 'info-mode-hook
;         '(lambda ()
;            (set-frame-property (buffer-dedicated-frame) [default
;                                                           font]
;                                "-*-Helvetica-Medium-R-*-*-*-120-*-*-*-*-*-*")
;            ))
;;}}}
;;----------------------------------------------------------------------------
;;{{{ R - Gnu's S
;;(ska-safe-require 'ess-site)
;;}}}
;;---------------------------------------------------------------------------- 
;;{{{ Povray-Mode
(setq auto-mode-alist
      (append
       '(("\\.pov$" . pov-mode))
       auto-mode-alist))
(autoload 'pov-mode "pov-mode"
  "Major mode for edition povray scene files" t nil)
;;}}}
;;----------------------------------------------------------------------------
;;{{{ Speedbar-Mode
(add-hook 'speedbar-mode-hook
          '(lambda ()
             (ska-speedbar-keys)))
;;}}}
;;----------------------------------------------------------------------------
;;{{{ MatLab Mode
(autoload 'matlab-mode "matlab" "Matlab Editing Mode" t)
(add-to-list
 'auto-mode-alist
 '("\\.m$" . matlab-mode))
(setq matlab-indent-function t)
(setq matlab-shell-command "matlab")
;; the original in matlab.el does not work for me :(
(copy-face 'paren-match 'matlab-region-face)
(add-hook 'matlab-mode-hook
          '(lambda ()
             (ska-matlab-mode-keys)
             ;; line numbers
             (turn-on-setnu-mode)
             ;; syntax highlighting
             (turn-on-font-lock)
             (make-local-hook 'write-contents-hooks)
             (add-hook 'write-contents-hooks #'ska-untabify
                       nil t)
             (setq indent-tabs-mode nil)
             ))
;;}}}
;;---------------------------------------------------------------------------- 
;;{{{ filedesc mode
(ska-safe-require 'filedesc-mode)
(add-hook 'filedesc-mode-hook
          '(lambda ()
             (turn-on-setnu-mode)
             (turn-on-font-lock)
             (auto-fill-mode nil)
             (setq indent-tabs-mode nil)
             (make-local-hook 'write-contents-hooks)
             (add-hook 'write-contents-hooks #'ska-untabify
                       nil t)))

;;---------------------------------------------------------------------------- 
;;{{{ EDIFACT mode
(ska-safe-require 'edifact-mode)
(add-hook 'edifact-mode-hook
          '(lambda ()
            (turn-on-font-lock)
            (auto-fill-mode nil)
            (edifact-break-lines t)
            ))

;;}}}
;;---------------------------------------------------------------------------- 
;;{{{ S2S Mode
(ska-safe-require 's2s-mode)
(add-hook 's2s-mode-hook
          '(lambda ()
            (turn-on-font-lock)
;            (make-local-hook 'write-contents-hooks)
;            (add-hook 'write-contents-hooks #'s2s-check-syntax
;                       nil t)
            ))
;;}}}
;;---------------------------------------------------------------------------- 
;;{{{ LinMag Mode
(ska-safe-require 'linmag-mode)
(add-hook 'linmag-mode-hook
          '(lambda ()
            (turn-on-font-lock)))

;;}}}
;;---------------------------------------------------------------------------- 
;;{{{ Mail Mode
(add-hook 'mail-setup-hook
          '(lambda ()
             (ska-mail-keys)
             ))
;;}}}
;;----------------------------------------------------------------------------
;;{{{ Makefiles
(add-hook 'makefile-mode-hook
          '(lambda ()
             (ska-makefile-keys)
             ))
;;}}}
;;----------------------------------------------------------------------------
;;{{{ PCL-CVS
;(setq cvs-inhibit-copyright-message t)
(add-hook 'cvs-mode-hook
      #'(lambda ()
;         (ska-cvs-keys)
          ))
;;}}}
;;----------------------------------------------------------------------------
;;}}}
;;============================================================================

;;============================================================================
;;{{{ LISP-PACKAGES

;;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;;----------------------------------------------------------------------------
;;{{{ Appointments
;; Has problems with a missing display-time-filer function
;;
;;(if (not (featurep 'appt))(require 'appt))
;;(appt-initialize)
;;(setq appt-msg-countdown-list '(10 5 1)
;;      appt-audible nil
;;      appt-check-time-syntax nil
;;      appt-announce-method 'appt-persistent-message-announce
;;      appt-display-duration 59)
;;(defun diary-save-hook ()
;;"Stuff to do when saving the diary files."
;;  (appt-initialize))
;;(defun add-diary-save-hook ()
;;  "find-file-hooks hook to add the diary-save-hook when appropriate"
;;  (if (string-match "diary" (buffer-name))
;;      (add-hook 'after-save-hook 'diary-save-hook)))
;;(add-hook 'find-file-hooks 'add-diary-save-hook)\
;;}}}
;;----------------------------------------------------------------------------
;;{{{ EUDC - Ldap++
;; Booh, this can actually _crash_ XEmacs!  Think, it needs more
;; work.  Somehow it works when I set it up manually but not from init
;; files?x
;;
;(setq ldap-host-parameters-alist
;      '(("comm-server.bn-software.local" base "o=Bernigshausen & Neben oHG")))

;(setq ldap-default-host "comm-server.bn-software.local")
;(setq ldap-default-base "o=Bernigshausen & Neben oHG")
;(setq ldap-ldapsearch-args '("-x" "-LL"))
;(setq eudc-inline-expansion-format '("%s %s <%s>" firstname name email))
;(setq eudc-strict-return-matches nil
;      eudc-inline-query-format '((name)
;                                 (firstname)
;                                 (firstname name)))
(setq eudc-inline-expansion-format '("%s <%s>" cn email))
;;}}}
;;----------------------------------------------------------------------------
;;{{{ TeX-Diary
;;(ska-safe-require 'tex-diary)
;;}}}
;;----------------------------------------------------------------------------
;;{{{ Swiss Movement
(ska-safe-require 'swiss-move)
;;}}}
;;----------------------------------------------------------------------------
;;{{{ MTorus
(defun mtorus-my-buffer-skip-p (buffer)
  "The my predicate used for `mtorus-buffer-skip-p'
is to skip only the special buffers whose name begins with a space."
  (or
   (string-match "^[ ]+"         (buffer-name buffer))
   (string-match "[Cc]ompletio" (buffer-name buffer))
   (string-match "slime-events" (buffer-name buffer))
   (string-match "SLIME Note" (buffer-name buffer))
   (string-match "inferior-lisp" (buffer-name buffer))
   (string-match "[Cc]ompil" (buffer-name buffer))))
(setq mtorus-buffer-skip-p 'mtorus-my-buffer-skip-p)

;; This is the code for the post-1.6-aera:
;(ska-safe-require 'mtorus)
;(ska-safe-require 'mtorus-state)
;(mtorus-universe-init)
;(mtorus-install-suggested-bindings-ska)
;(mtorus-enable-select-resurrects-dead)
;(add-hook 'kill-emacs-hook
;          (lambda ()
;            (mtorus-state-save)))
;;}}}
;;----------------------------------------------------------------------------
;;{{{ TAB free coding
(defun ska-untabify ()
  "My untabify function as discussed and described at
http://www.jwz.org/doc/tabs-vs-spaces.html
and improved by Claus Brunzema:
 - return nil to get `write-contents-hooks' to work correctly
   (see documentation there)
 - `make-local-hook' instead of `make-local-variable'
 - when instead of if
Use some lines along the following for getting this to work in the
modes you want it to:
 
\(add-hook 'some-mode-hook  
          '(lambda () 
              (make-local-hook 'write-contents-hooks) 
               (add-hook 'write-contents-hooks 'ska-untabify nil t)))"
  (save-excursion
    (goto-char (point-min))
    (when (search-forward "\t" nil t)
      (untabify (1- (point)) (point-max)))
    nil))
;;}}}
;;----------------------------------------------------------------------------
;;{{{ Interface to CDargs
(ska-safe-require 'cdargs)
(add-hook 'cdargs-warped-hook
          '(lambda ()
             (when (file-exists-p
                    (expand-file-name desktop-basefilename "./"))
               (desktop-read))))
;;}}}
;;----------------------------------------------------------------------------
;;{{{ Scrolling Using Mouse Dragging
(ska-safe-require 'track-scroll)
;;}}}
;;----------------------------------------------------------------------------
;;{{{ Emacs Lisp Documentation
(autoload 'turn-on-eldoc-mode "eldoc" nil t)
;; this works not with standard eldoc.el from SuSE 7.3:
(setq eldoc-minor-mode-string " ed")
;; 
(setq eldoc-idle-delay 1.5)
;; is put into the hooks above...
;;}}}
;;----------------------------------------------------------------------------
;;{{{ MailCrypt
(load-library "mailcrypt") ; provides "mc-setversion"
(mc-setversion "gpg")
(autoload 'mc-install-write-mode "mailcrypt" nil t)
(autoload 'mc-install-read-mode "mailcrypt" nil t)
(add-hook 'mail-mode-hook 'mc-install-write-mode)
;;}}}
;;----------------------------------------------------------------------------
;;{{{ Auto Save
;; for auto-save-directory see top of file where all the special files
;; and dirs are defined
(setq auto-save-timeout 120)
(setq auto-save-interval 100)
;;}}}
;;----------------------------------------------------------------------------
;;{{{ Low Level Debug
;;this is required by ska-global-keys.el
;;(require 'll-debug)
;; need this for printf output in C++ mode
(setcdr (assq 'c++-mode ll-debug-insert-variable-output-alist)
        (cdr
         (assq 'c-mode ll-debug-insert-variable-output-alist)))

(setcdr (assq 'c++-mode ll-debug-insert-debug-output-alist)
        (cdr
         (assq 'c-mode ll-debug-insert-debug-output-alist)))

;;}}}
;;----------------------------------------------------------------------------
;;{{{ QueryReplaceByExample
(autoload 'query-replace-by-example "query-replace-by-example"
  "Get replace string by example." t)
;;}}}
;;----------------------------------------------------------------------------
;;{{{ Sawfish 
(ska-safe-require 'sawfish)
(setq auto-mode-alist
      (append
       (list
        '("\\.jl" . sawfish-mode))
       auto-mode-alist))
;;}}}
;;----------------------------------------------------------------------------
;;{{{ Marky Mode -- doesn't work correctly yet...
;; (require 'marky)
;;}}}
;;----------------------------------------------------------------------------
;;{{{ Highlight Scrolling Context
(load "highlight-context-line")
;; for old version: (setq highlight-context-line-expire 3)
(setq highlight-context-line-background "#c5c500")
;;}}}
;;----------------------------------------------------------------------------
;;{{{ Setnu: Line Numbers
;; please note that I have experienced some problems with this which
;; are not trackable for me. When you get some kind of extent-error
;; when killing lines or creating new lines or something else, try
;; turning setnu off by calling M-x setnu-mode and see if that helps
;; or reopen the file you are just editing. Otherwise XEmacs might
;; crash soon :(
(setq setnu-line-number-format "%4d ")
(setq setnu-line-number-face (copy-face 'default 'setnu-my-face))
(set-face-background 'setnu-my-face "#bfbfbf")
(set-face-foreground 'setnu-my-face "#777777")
;;}}}
;;----------------------------------------------------------------------------
;;{{{ Active Menu
;; The menubar is only shown when the mouse comes to the top of the window
;; my idea, claus' implementation :)
;; see http://www.cbrunzema.de/software.html#active-menu
(ska-safe-require 'active-menu)
(setq active-menu-sensitive-height 10)
(turn-on-active-menu)
;;}}}
;;----------------------------------------------------------------------------
;;{{{ Locate
;; An Emacs interface to the unix locate command
;; taken from the GNU Emacs distribution from Debian (CHB)
(autoload 'locate "locate")
(autoload 'locate-with-filter "locate")
;;}}}
;;----------------------------------------------------------------------------
;;{{{ folding
;; hide and show blocks 
(load "folding" 'nomessage 'noerror)
(folding-mode-add-find-file-hook)
;;}}}
;;----------------------------------------------------------------------------
;;{{{ Function Menu
;;parse functions for easy navigation
(ska-safe-require 'func-menu)
;; Addition to func-menu: the ruby stuff
;; NOTE: I tried to patch the original but got confused which version
;; loaded at starttime and such....
(defvar fume-function-name-regexp-ruby
  "^\\s-*\\(class\\|def\\)+\\s-*\\([^(\n ]+\\)")
(setq fume-function-name-regexp-alist
      (append
       '((ruby-mode . fume-function-name-regexp-ruby))
       fume-function-name-regexp-alist))

(defun fume-find-next-ruby-function-name (buffer)
  "Searches for the next ruby function in BUFFER."
  (set-buffer buffer)
  (if (re-search-forward fume-function-name-regexp nil t)
      (save-excursion
        (let* ((retpnt (match-beginning 2))
               (retname (buffer-substring retpnt (match-end 2))))
          (goto-char (match-beginning 0))
          (cond ((looking-at "^\\s-*def")
                 (if (re-search-backward
                  "\n?\\s-*class\\s-*\\([A-Z][A-Za-z0-9_]*\\)\\s-*" nil t)
                 (let* ((classname (buffer-substring
                                    (match-beginning 1) (match-end 1))))
                   (if (not (string-match (concat "^" classname "\\.") retname))
                       (setq retname (concat classname "." retname)))))))
          (cons retname retpnt)))))


(setq fume-find-function-name-method-alist
      (append
       '((ruby-mode . fume-find-next-ruby-function-name))
       fume-find-function-name-method-alist))

;;}}}
;;----------------------------------------------------------------------------
;;{{{ Recent Files/Picklist
;;Show last visited files in menubar


;;(load "recent-files")
;;(setq recent-files-dont-include
;;      '("~$" "tmp/." "contact/." "INBOX" ".bbdb" ".newsrc." "places.el"))
;; moved to customize:
;;(setq recent-files-non-permanent-submenu t)
;;(setq recent-files-commands-submenu t)
;;(setq recent-files-number-of-entries 40)
;;(recent-files-initialize)

;; An Alternative from http://www.cbrunzema.de/software.html
(ska-safe-require 'picklist)
(setq picklist-permanent-submenu t)
(setq picklist-commands-submenu t)
(setq picklist-number-of-menu-entries 50)
(setq picklist-dont-include 
      '("~$" "tmp/." "contact/." "INBOX" ".bbdb" ".newsrc."
      "places.el"))
(setq picklist-sort-function #'picklist-sort-atime)
(setq picklist-add-menu-before "Edit")
(picklist-initialize)
;;}}}
;;----------------------------------------------------------------------------
;;{{{ filladapt
(setq filladapt-mode-line-string " Fa")
;;}}}
;;----------------------------------------------------------------------------
;;{{{ vertical scrolling in place 
;; unfortunately this messes up my VM, so until I found out how to
;; clean that, it stays commented out
;;(ska-safe-require 'scroll-in-place)
;;(turn-on-scroll-in-place)
;;}}}
;;----------------------------------------------------------------------------
;;{{{ save cursorpositions
(ska-safe-require 'saveplace)
(setq-default save-place t)
(setq save-place-limit 100)
;;}}}
;;----------------------------------------------------------------------------
;;{{{ hippie-expand
;;expand text trying various ways to find its expansion.

(ska-safe-require 'hippie-exp)
(setq hippie-expand-try-functions-list
      '(try-expand-dabbrev
        try-expand-dabbrev-all-buffers
        try-complete-file-name-partially
        try-expand-list
        try-complete-lisp-symbol-partially
        try-expand-line))
;;}}}
;;----------------------------------------------------------------------------
;;{{{ bbdb
;; the insidous big brother data base -> put into vm-init-file
;;(ska-safe-require 'bbdb)
;;(bbdb-initialize 'vm 'message 'sendmail)
(setq bbdb/news-auto-create-p 'bbdb-ignore-most-messages-hook
      bbdb/mail-auto-create-p 'bbdb-ignore-most-messages-hook
      bbdb-ignore-most-messages-alist 
      '(("To" . "skamphausen@bn-software\\.de")
        ("Subject" . "\\(emacs\\|cdargs\\|cl-ipc\\|Web-Kontakt\\)"))
      bbdb-ignore-some-messages-alist
      '(("From" . "daemon")
        ("From" . "abuse@")
        ("From" . "news@")
        ("From" . "cron")
        ("From" . "root@")
        ("From" . "noreply@")
        ("From" . "prontomail")
        ("From" . "majordomo")
        ("From" . "postmaster")
        ("From" . "quota")
        ("Subject" . "MAILER DAEMON")))
(setq bbdb-notice-hook 'bbdb-auto-notes-hook)
(setq bbdb-auto-notes-alist
      '(("Organization" (".*" company 0))
        ("Newsgroup" ("[^,]+" newsgroups 0))
        ("Subject" (".*" last-subj 0 t))
        ("User-Agent" (".*" mailer 0))
        ("X-Newsreader" (".*" mailer 0))
        ("X-gpg-key-ID" (".*" key 0))
        ("X-Mailer" (".*" mailer 0))
        ("X-URL" (".*" url 0))
        ;;("X-Face" (".+" face 0 'replace))
        ;;("Face" (".+" cface 0 'replace))
        ))
(setq bbdb-default-area-code "0551"
            bbdb-north-american-phone-numbers-p nil
            bbdb-complete-name-allow-cycling t)
;;}}}
;;----------------------------------------------------------------------------
;;{{{ letter-template
;(setq texletter-template-directory "~/text/tex/LetterTemplates")
;(setq texletter-letter-directory   "~/text/tex")
;(autoload 'texletter-create-letter
;  "texletter"
;  "Create LaTeX letters from templates."
;  t)
;(autoload 'texletter-install-templates-as-menu
;  "texletter"
;  "Adding menu items for all templates."
;  t)
;(texletter-install-templates-as-menu)
;;}}}
;;----------------------------------------------------------------------------
;;{{{ resize-minibuffer-mode
;; makes the minibuffer automatically resize when _typing_ text. This
;; is not for displaying larger information
(autoload 'resize-minibuffer-mode "rsz-minibuf" nil t)
(resize-minibuffer-mode 1)
(setq resize-minibuffer-window-exactly nil)
;;}}}
;;----------------------------------------------------------------------------
;;{{{ minibuffer-complete: Cycle completions in minibuffer
(load "minibuffer-complete-cycle")
;;}}}
;;----------------------------------------------------------------------------
;;{{{ Speedbar
;(autoload 'speedbar-frame-mode "speedbar" "Popup a speedbar frame" t) 
;(autoload 'speedbar-get-focus "speedbar" "Jump to speedbar frame" t)
(require 'speedbar)
(speedbar-add-supported-extension ".lisp")
;;}}}
;;----------------------------------------------------------------------------
;;{{{ AutoInsert
;; automatic insertion of text into new files.
;; Find appropriate skeletons for various modes.
(load "ska-auto-insert" nil nil nil)
;;}}}
;;----------------------------------------------------------------------------
;;{{{ Jiggle The Cursor
;; http://www.eskimo.com/~seldon/jiggle.el
;; I changed the modeline minor mode marker in the .el file
;; Problem is that this plugs into the wrong hook. It is called
;; whenever a buffer is selected but I want it to only jiggle when the
;; buffer is shown (compare chb-next-buffer defun)
;(require 'jiggle)
;(setq jiggle-sit-for-how-long .05)
;(setq jiggle-how-many-times 2)
;;}}}
;;----------------------------------------------------------------------------
;;{{{ counter
;; http://www.eskimo.com/~seldon/counter.el
;;(autoload 'counter "counter" nil t)
;;}}}
;;----------------------------------------------------------------------------
;;{{{ ispell

;; Es gibt in Version >20 ein Problem mit ispell, dass eine Funktion
;; nicht definiert ist :-(
;; Dieses soll wohl ersma laufen:
;(if (not (fboundp 'set-process-coding-system))
;    (defun set-process-coding-system (foo bar)
;      (print "Still an ispell error")
;     ))
;; which dictionary
(setq-default ispell-dictionary "american")
;;}}}
;;----------------------------------------------------------------------------
;;{{{ gnuserv
;; Xemacs acts as a server for editing sessions
;; (unless (gnuserv-running-p) (gnuserv-start))
;;}}}
;;----------------------------------------------------------------------------
;;{{{ efs
;; remote file editing
;(ska-safe-require 'efs)
;(setq efs-default-user "username")
;(efs-set-user "ftp.yourdomain.org" "yourusername")
;(efs-display-ftp-activity)
;;}}}
;;----------------------------------------------------------------------------
;;{{{ vkill
(autoload 'vkill "vkill" nil t)
(autoload 'list-unix-processes "vkill" nil t)
;;}}}
;;----------------------------------------------------------------------------
;;{{{ fff
(autoload 'fff-find-file-in-envvar-path                    "fff" nil t)
(autoload 'fff-insert-file-in-envvar-path                  "fff" nil t)
(autoload 'fff-find-file-in-exec-path                      "fff" nil t)
(autoload 'fff-insert-file-in-exec-path                    "fff" nil t)
(autoload 'fff-find-file-in-path                           "fff" nil t)
(autoload 'fff-insert-file-in-path                         "fff" nil t)
(eval-after-load "fff" '(fff-install-map))
;;}}}
;;----------------------------------------------------------------------------
;;}}}
;;============================================================================

;;============================================================================
;;{{{ MISCELLEANOUS SETTINGS
;;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;;{{{ Important Hooks
(add-hook 'after-save-hook
        #'(lambda ()
        (and (save-excursion
               (save-restriction
                 (widen)
                 (goto-char (point-min))
                 (save-match-data
                   (looking-at "^#!"))))
             (not (file-executable-p buffer-file-name))
             (shell-command (concat "chmod u+x " (shell-quote-argument
                                                  buffer-file-name)))
             (message
              (concat "Saved as script: " buffer-file-name)))))
;;}}}
;;----------------------------------------------------------------------------
;;{{{ tiny stuff 
;; no splash screen
(setq-default inhibit-startup-message t)
;; y instead of  y e s  is enough
(fset 'yes-or-no-p 'y-or-n-p)
;; delete key != backspace
(setq delete-key-deletes-forward t)
;; file name in icon
(setq frame-title-format (list mode-line-buffer-identification))
;; Timestamp within the first `time-stamp-line-limit' lines
;; Format: Time-stamp: <>
(setq time-stamp-active t)
(setq time-stamp-format "%02d-%3b-%:y %02H:%02M:%02S %u")
(setq time-stamp-line-limit 16)
;; fr Menueintrag Edit->Text Properties
(ska-safe-require 'facemenu)
;; lade SVN frontend
(ska-safe-require 'psvn)
;; browser 
;;(setq-default browse-url-browser-function 'browse-url-netscape)
;; The default mode should not just be fundamental
(setq default-major-mode 'text-mode)
;; flash only top and bottom line
(setq visible-bell 'top-bottom)
;; allow font locking for even larger buffers (nil would allow each
;; size)
(setq font-lock-maximum-size 400000)
;; No, thanks, I don't need that blurb in the scratch buffer
(setq initial-scratch-message ";; * SCRATCH *\n")
;;}}}
;;----------------------------------------------------------------------------
;;{{{ other variables
;; ignore files when completing with TAB
(setq completion-ignored-extensions 
      (append '(".aux" ".dvi" ".ps" ".bbl" ".blg" ".o" ".class"
      ".fasl" ".x86f") 
              completion-ignored-extensions)
      )

;; Tune Modeline To My Taste:
;; =========================
;;
;; The Problem is that modes that change the modeline rely on it to be
;; in the original format (e.g. fume-toggle-modeline-display)
;;
;; line and column number before file name (so that they don't get
;; lost for long fume-displays)
;; and remove the "Xemacs:"-string...
;;(defvar my-modeline-buffer-identification
;;  (list (cons modeline-buffer-id-left-extent ":")
;;        (cons modeline-buffer-id-right-extent " %17b"))
;;  "My modification of `modeline-buffer-identification': removed the
;;  \"XEmacs\" from it.")
;; if I remove XEmacs, fume doesn't work anymore :(
;(setq default-modeline-format
;      (list
;       ""   
;       (cons modeline-modified-extent 'modeline-modified)
;       (list 'line-number-mode "(%l,")
;       (list 'column-number-mode "%c) ")
;       (cons modeline-buffer-id-extent 'modeline-buffer-identification)
;       "  "
;       'global-mode-string
;       "   %[("
;       (cons modeline-minor-mode-extent
;             (list "" 'mode-name 'minor-mode-alist))
;       (cons modeline-narrowed-extent "%n")
;       'modeline-process
;       ")%]---"
;       (cons -3 "%p")
;       "-%-"))
;;}}}
;;----------------------------------------------------------------------------
(require 'mwheel)
(mwheel-install)


;;}}}
;;============================================================================


;; Local variables:
;; mode: emacs-lisp
;; folded-file: nil
;; end:

